@@ -1,8 +1,8 @@
 require 'ffi-glib/container_class_methods'
 
-module GLib
-  load_class :HashTable
+GLib.load_class :HashTable
 
+module GLib
   # Overrides for GHashTable, GLib's hash table implementation.
   class HashTable
     include Enumerable
@@ -12,27 +12,26 @@ module GLib
     attr_reader :value_type
 
     def each
-      prc = Proc.new {|keyptr, valptr, userdata|
+      prc = proc {|keyptr, valptr, _userdata|
         key = GirFFI::ArgHelper.cast_from_pointer key_type, keyptr
         val = GirFFI::ArgHelper.cast_from_pointer value_type, valptr
         yield key, val
       }
-      ::GLib::Lib.g_hash_table_foreach self.to_ptr, prc, nil
+      callback = GLib::HFunc.from prc
+      ::GLib::Lib.g_hash_table_foreach to_ptr, callback, nil
     end
 
     def to_hash
-      Hash[self.to_a]
+      Hash[to_a]
     end
 
     def insert key, value
       keyptr = GirFFI::InPointer.from key_type, key
       valptr = GirFFI::InPointer.from value_type, value
-      ::GLib::Lib.g_hash_table_insert self.to_ptr, keyptr, valptr
+      ::GLib::Lib.g_hash_table_insert to_ptr, keyptr, valptr
     end
 
-    class << self
-      remove_method :new
-    end
+    class << self; remove_method :new; end
 
     def self.new keytype, valtype
       wrap [keytype, valtype], Lib.g_hash_table_new(
@@ -45,7 +44,7 @@ module GLib
     end
 
     def self.from_enumerable typespec, hash
-      ghash = self.new(*typespec)
+      ghash = new(*typespec)
       hash.each do |key, val|
         ghash.insert key, val
       end
