@@ -1,8 +1,8 @@
 module GLib
   # Extra methods for GLib::Strv. The bulk is defined in `gir_ffi-base/glib/strv.rb`
   class Strv
-    def ==(other)
-      self.to_a == other.to_a
+    def == other
+      to_a == other.to_a
     end
 
     def self.from it
@@ -19,7 +19,7 @@ module GLib
     end
 
     def self.from_enumerable enum
-      self.wrap GirFFI::InPointer.from_array :utf8, enum
+      wrap GirFFI::InPointer.from_array :utf8, enum
     end
   end
 end
