@@ -5,7 +5,8 @@ module GObjectIntrospection
     def n_values
       Lib.g_enum_info_get_n_values @gobj
     end
-    def value(index)
+
+    def value index
       IValueInfo.wrap(Lib.g_enum_info_get_value @gobj, index)
     end
     ##
@@ -14,7 +15,8 @@ module GObjectIntrospection
     def get_n_methods
       Lib.g_enum_info_get_n_methods @gobj
     end
-    def get_method(index)
+
+    def get_method index
       IFunctionInfo.wrap(Lib.g_enum_info_get_method @gobj, index)
     end
 
@@ -27,4 +29,3 @@ module GObjectIntrospection
     end
   end
 end
-
