@@ -22,46 +22,46 @@ require 'ffi-gobject_introspection/i_union_info'
 require 'ffi-gobject_introspection/i_enum_info'
 require 'ffi-gobject_introspection/i_flags_info'
 
-module GObjectIntrospection
-  GObject.type_init
+GObject.type_init
 
+module GObjectIntrospection
   # The Gobject Introspection Repository. This class is the point of
   # access to the introspection typelibs.
   # This class wraps the GIRepository struct.
   class IRepository
     # Map info type to class. Default is IBaseInfo.
     TYPEMAP = {
-      :invalid => IBaseInfo,
-      :function => IFunctionInfo,
-      :callback => ICallbackInfo,
-      :struct => IStructInfo,
+      invalid:                  IBaseInfo,
+      function:                 IFunctionInfo,
+      callback:                 ICallbackInfo,
+      struct:                   IStructInfo,
       # TODO: There's no GIBoxedInfo, so what does :boxed mean?
-      :boxed => IBaseInfo,
-      :enum => IEnumInfo,
-      :flags => IFlagsInfo,
-      :object => IObjectInfo,
-      :interface => IInterfaceInfo,
-      :constant => IConstantInfo,
-      :invalid_was_error_domain => IBaseInfo,
-      :union => IUnionInfo,
-      :value => IValueInfo,
-      :signal => ISignalInfo,
-      :vfunc => IVFuncInfo,
-      :property => IPropertyInfo,
-      :field => IFieldInfo,
-      :arg => IArgInfo,
-      :type => ITypeInfo,
-      :unresolved => IBaseInfo
+      boxed:                    IBaseInfo,
+      enum:                     IEnumInfo,
+      flags:                    IFlagsInfo,
+      object:                   IObjectInfo,
+      interface:                IInterfaceInfo,
+      constant:                 IConstantInfo,
+      invalid_was_error_domain: IBaseInfo,
+      union:                    IUnionInfo,
+      value:                    IValueInfo,
+      signal:                   ISignalInfo,
+      vfunc:                    IVFuncInfo,
+      property:                 IPropertyInfo,
+      field:                    IFieldInfo,
+      arg:                      IArgInfo,
+      type:                     ITypeInfo,
+      unresolved:               IBaseInfo
     }
 
     def initialize
-      @gobj = Lib::g_irepository_get_default
+      @gobj = Lib.g_irepository_get_default
     end
 
     include Singleton
 
     def self.default
-      self.instance
+      instance
     end
 
     def self.prepend_search_path path
@@ -86,8 +86,7 @@ module GObjectIntrospection
     end
 
     def info namespace, index
-      ptr = Lib.g_irepository_get_info @gobj, namespace, index
-      return wrap ptr
+      wrap_info Lib.g_irepository_get_info(@gobj, namespace, index)
     end
 
     # Utility method
@@ -98,14 +97,12 @@ module GObjectIntrospection
     end
 
     def find_by_name namespace, name
-      ptr = Lib.g_irepository_find_by_name @gobj, namespace, name
-      return wrap ptr
+      wrap_info Lib.g_irepository_find_by_name(@gobj, namespace, name)
     end
 
     def find_by_gtype gtype
       raise ArgumentError, "Type #{gtype} is not a valid type" if gtype == 0
-      ptr = Lib.g_irepository_find_by_gtype @gobj, gtype
-      return wrap ptr
+      wrap_info Lib.g_irepository_find_by_gtype(@gobj, gtype)
     end
 
     def dependencies namespace
@@ -120,17 +117,15 @@ module GObjectIntrospection
 
     def self.wrap_ibaseinfo_pointer ptr
       return nil if ptr.null?
-
       type = Lib.g_base_info_get_type ptr
       klass = TYPEMAP[type]
-
-      return klass.wrap(ptr)
+      klass.wrap ptr
     end
 
     private
 
-    def wrap ptr
-      IRepository.wrap_ibaseinfo_pointer ptr
+    def wrap_info ptr
+      self.class.wrap_ibaseinfo_pointer ptr
     end
   end
 end
