@@ -5,7 +5,8 @@ module GObjectIntrospection
     def n_fields
       Lib.g_struct_info_get_n_fields @gobj
     end
-    def field(index)
+
+    def field index
       IFieldInfo.wrap(Lib.g_struct_info_get_field @gobj, index)
     end
 
@@ -16,7 +17,8 @@ module GObjectIntrospection
     def get_n_methods
       Lib.g_struct_info_get_n_methods @gobj
     end
-    def get_method(index)
+
+    def get_method index
       IFunctionInfo.wrap(Lib.g_struct_info_get_method @gobj, index)
     end
 
