@@ -1,6 +1,7 @@
 require 'ffi'
 
 module GObjectIntrospection
+  # Module for attaching functions from the girepository library
   module Lib
     extend FFI::Library
     ffi_lib "girepository-1.0"
@@ -121,8 +122,6 @@ module GObjectIntrospection
     # improved signature.
     attach_function :g_type_tag_to_string, [:ITypeTag], :string
 
-    #define G_TYPE_TAG_IS_BASIC(tag) (tag < GI_TYPE_TAG_ARRAY)
-
     enum :IArrayType, [
       :c,
       :array,
@@ -210,13 +209,7 @@ module GObjectIntrospection
 
     # IVFuncInfo
 
-    enum :IVFuncInfoFlags, [
-      :must_chain_up, (1 << 0),
-      :must_override, (1 << 1),
-      :must_not_override, (1 << 2)
-    ]
-
-    attach_function :g_vfunc_info_get_flags, [:pointer], :IVFuncInfoFlags
+    attach_function :g_vfunc_info_get_flags, [:pointer], :int
     attach_function :g_vfunc_info_get_offset, [:pointer], :int
     attach_function :g_vfunc_info_get_signal, [:pointer], :pointer
     attach_function :g_vfunc_info_get_invoker, [:pointer], :pointer
