@@ -32,12 +32,12 @@ module GirFFI
     end
 
     def setup_constants
-      @klass.const_set :GIR_INFO, info
-      @klass.const_set :GIR_FFI_BUILDER, self
+      klass.const_set :GIR_INFO, info
+      klass.const_set :GIR_FFI_BUILDER, self
     end
 
     def already_set_up
-      const_defined_for @klass, :GIR_FFI_BUILDER
+      klass.const_defined? :GIR_FFI_BUILDER, false
     end
 
     def gir
