@@ -1,5 +1,4 @@
 require 'gir_ffi/builders/registered_type_builder'
-require 'gir_ffi/builders/with_methods'
 require 'gir_ffi/enum_base'
 
 module GirFFI
@@ -8,8 +7,6 @@ module GirFFI
     # attached to the appropriate namespace module, and will be defined
     # as an enum for FFI.
     class EnumBuilder < RegisteredTypeBuilder
-      include WithMethods
-
       private
 
       def enum_sym
@@ -17,28 +14,32 @@ module GirFFI
       end
 
       def value_spec
-        return info.values.map {|vinfo|
+        info.values.map {|vinfo|
           val = GirFFI::ArgHelper.cast_uint32_to_int32(vinfo.value)
           [vinfo.name.to_sym, val]
         }.flatten
       end
 
       def instantiate_class
-        @klass = get_or_define_module namespace_module, @classname
-        @enum = optionally_define_constant @klass, :Enum do
+        @enum = optionally_define_constant klass, :Enum do
           lib.enum(enum_sym, value_spec)
         end
-        unless already_set_up
-          @klass.extend superclass
-          setup_constants
-          setup_gtype_getter
-          stub_methods
-          setup_inspect
-        end
+        setup_class unless already_set_up
+      end
+
+      def setup_class
+        klass.extend superclass
+        setup_constants
+        stub_methods
+        setup_inspect
+      end
+
+      def klass
+        @klass ||= get_or_define_module namespace_module, @classname
       end
 
       def setup_inspect
-        @klass.instance_eval <<-EOS
+        klass.instance_eval <<-EOS
           def self.inspect
             "#{@namespace}::#{@classname}"
           end
@@ -46,17 +47,12 @@ module GirFFI
       end
 
       def already_set_up
-        @klass.respond_to? :get_gtype
+        klass.respond_to? :get_gtype
       end
 
       def superclass
         @superclass ||= EnumBase
       end
-
-      # FIXME: Remove duplication with ModuleBuilder
-      def get_or_define_module parent, name
-        optionally_define_constant(parent, name) { ::Module.new }
-      end
     end
   end
 end
