@@ -1,7 +1,7 @@
 require 'gir_ffi/builders/argument_builder'
 require 'gir_ffi/builders/return_value_builder'
 require 'gir_ffi/variable_name_generator'
-require 'gir_ffi/setter_argument_info'
+require 'gir_ffi/field_argument_info'
 
 module GirFFI
   module Builders
@@ -14,10 +14,24 @@ module GirFFI
       end
 
       def build
-        unless container_info.find_instance_method info.name
-          container_class.class_eval getter_def
-        end
-        container_class.class_eval setter_def if info.writable?
+        setup_getter
+        setup_setter
+      end
+
+      def setup_getter
+        container_class.class_eval getter_def unless container_defines_getter_method?
+      end
+
+      def container_defines_getter_method?
+        container_info.find_instance_method info.name
+      end
+
+      def setup_setter
+        container_class.class_eval setter_def if is_writable_field?
+      end
+
+      def is_writable_field?
+        info.writable?
       end
 
       def getter_def
@@ -26,13 +40,13 @@ module GirFFI
         field_ptr = builder.new_variable
         typed_ptr = builder.new_variable
 
-        return <<-CODE.reset_indentation
+        <<-CODE.reset_indentation
         def #{info.name}
           #{field_ptr} = @struct.to_ptr + #{info.offset}
           #{typed_ptr} = GirFFI::InOutPointer.new(#{field_type_tag_or_class.inspect}, #{field_ptr})
-          #{builder.callarg} = #{typed_ptr}.to_value
-          #{builder.post.join("\n")}
-          #{builder.retval}
+          #{builder.capture_variable_name} = #{typed_ptr}.to_value
+          #{builder.post_conversion.join("\n")}
+          #{builder.return_value_name}
         end
         CODE
       end
@@ -43,11 +57,11 @@ module GirFFI
         field_ptr = builder.new_variable
         typed_ptr = builder.new_variable
 
-        return <<-CODE.reset_indentation
-        def #{info.name}= #{builder.inarg}
+        <<-CODE.reset_indentation
+        def #{info.name}= #{builder.method_argument_name}
           #{field_ptr} = @struct.to_ptr + #{info.offset}
           #{typed_ptr} = GirFFI::InOutPointer.new(#{field_type_tag_or_class.inspect}, #{field_ptr})
-          #{builder.pre.join("\n          ")}
+          #{builder.pre_conversion.join("\n          ")}
           #{typed_ptr}.set_value #{builder.callarg}
         end
         CODE
@@ -79,15 +93,22 @@ module GirFFI
         @info.name.to_sym
       end
 
+      def field_type
+        @field_type ||= @info.field_type
+      end
+
+      def field_argument_info
+        @field_argument_info ||= FieldArgumentInfo.new "value", field_type
+      end
+
       def return_value_builder
-        vargen = VariableNameGenerator.new
-        @rv_builder ||= ReturnValueBuilder.new vargen, @info.field_type
+        @rv_builder ||= ReturnValueBuilder.new(VariableNameGenerator.new,
+                                               field_argument_info)
       end
 
       def setter_builder
-        vargen = VariableNameGenerator.new
-        argument_info = SetterArgumentInfo.new "value", @info.field_type
-        ArgumentBuilder.new vargen, argument_info
+        @setter_builder ||= ArgumentBuilder.new(VariableNameGenerator.new,
+                                                field_argument_info)
       end
     end
   end
