@@ -1,6 +1,5 @@
 require 'gir_ffi/builders/registered_type_builder'
 require 'gir_ffi/builders/with_layout'
-require 'gir_ffi/builders/with_methods'
 require 'gir_ffi/builders/property_builder'
 require 'gir_ffi/object_base'
 
@@ -8,7 +7,6 @@ module GirFFI
   module Builders
     # Implements the creation of a class representing a GObject Object.
     class ObjectBuilder < RegisteredTypeBuilder
-      include WithMethods
       include WithLayout
 
       def find_signal signal_name
@@ -26,13 +24,16 @@ module GirFFI
           raise "Property #{property_name} not found"
       end
 
+      def object_class_struct
+        @object_class_struct ||= Builder.build_class object_class_struct_info
+      end
+
       private
 
       def setup_class
         setup_layout
         setup_constants
         stub_methods
-        setup_gtype_getter
         if info.fundamental?
           setup_field_accessors
         else
@@ -67,21 +68,12 @@ module GirFFI
                         end
       end
 
-      # TODO: Unify with field accessor setup.
       def setup_property_accessors
         info.properties.each do |prop|
-          setup_accessors_for_property_info prop
+          PropertyBuilder.new(prop).build
         end
       end
 
-      def setup_accessors_for_property_info prop
-        builder = Builder::Property.new prop
-        unless info.find_instance_method prop.getter_name
-          @klass.class_eval builder.getter_def
-        end
-        @klass.class_eval builder.setter_def
-      end
-
       # TODO: Guard agains accidental invocation of undefined vfuncs.
       # TODO: Create object responsible for creating these invokers
       def setup_vfunc_invokers
@@ -94,7 +86,7 @@ module GirFFI
 
       def define_vfunc_invoker vfunc_name, invoker_name
         return if vfunc_name == invoker_name
-        @klass.class_eval "
+        klass.class_eval "
           def #{vfunc_name} *args, &block
             #{invoker_name}(*args, &block)
           end
@@ -103,9 +95,7 @@ module GirFFI
 
       def setup_interfaces
         interfaces.each do |iface|
-          @klass.class_eval do
-            include iface
-          end
+          klass.send :include, iface
         end
       end
 
@@ -118,6 +108,10 @@ module GirFFI
           GirFFI::Builder.build_class ifinfo
         end
       end
+
+      def object_class_struct_info
+        @object_class_struct_info ||= info.class_struct
+      end
     end
   end
 end
