@@ -4,9 +4,10 @@ require 'gir_ffi/builders/enum_builder'
 require 'gir_ffi/builders/interface_builder'
 require 'gir_ffi/builders/object_builder'
 require 'gir_ffi/builders/struct_builder'
-require 'gir_ffi/builders/signal_builder'
+require 'gir_ffi/builders/signal_closure_builder'
 require 'gir_ffi/builders/unintrospectable_builder'
 require 'gir_ffi/builders/union_builder'
+require 'gir_ffi/builders/vfunc_builder'
 
 module GirFFI
   module Builders
@@ -16,15 +17,15 @@ module GirFFI
       CACHE = {}
 
       TYPE_MAP = {
-        :callback => CallbackBuilder,
-        :constant => ConstantBuilder,
-        :enum => EnumBuilder,
-        :flags => EnumBuilder,
-        :interface => InterfaceBuilder,
-        :object => ObjectBuilder,
-        :struct => StructBuilder,
-        :union => UnionBuilder,
-        :unintrospectable => UnintrospectableBuilder
+        callback:         CallbackBuilder,
+        constant:         ConstantBuilder,
+        enum:             EnumBuilder,
+        flags:            EnumBuilder,
+        interface:        InterfaceBuilder,
+        object:           ObjectBuilder,
+        struct:           StructBuilder,
+        union:            UnionBuilder,
+        unintrospectable: UnintrospectableBuilder
       }
 
       def self.build info
