@@ -7,21 +7,20 @@ module GirFFI
     # cast objects returned by a function that returns an interface.
     class UnintrospectableBuilder < ObjectBuilder
       def instantiate_class
-        gtype = target_gtype
-        TypeBuilder::CACHE[gtype] ||= Class.new(superclass)
-        @klass = TypeBuilder::CACHE[gtype]
-        @structklass = get_or_define_class @klass, :Struct, layout_superclass
         setup_class unless already_set_up
       end
 
+      def klass
+        @klass ||= TypeBuilder::CACHE[target_gtype] ||= Class.new(superclass)
+      end
+
       def setup_class
         setup_constants
         setup_layout
         setup_interfaces
-        setup_gtype_getter
       end
 
-      def setup_instance_method method
+      def setup_instance_method _method
         false
       end
 
