@@ -4,8 +4,7 @@ module GirFFI
   module Builders
     # Implements the creation of classes representing types with layout,
     # i.e., :union, :struct, :object.
-    # Note: This module depends on the additional inclusion of
-    # WithMethods.
+    # Note: This module depends on methods in RegisteredTypeBuilder.
     module WithLayout
       def layout_specification
         spec = base_layout_specification
@@ -20,7 +19,7 @@ module GirFFI
 
       def setup_layout
         spec = layout_specification
-        @structklass.class_eval { layout(*spec) }
+        struct_class.class_eval { layout(*spec) }
       end
 
       def dummy_layout_specification
@@ -35,21 +34,23 @@ module GirFFI
         fields.map { |finfo| finfo.layout_specification }.flatten(1)
       end
 
-      def setup_accessors_for_field_info finfo
-        FieldBuilder.new(finfo).build
-      end
-
       def setup_field_accessors
         fields.each do |finfo|
-          setup_accessors_for_field_info finfo
+          FieldBuilder.new(finfo).build
         end
       end
 
       def instantiate_class
-        @klass = get_or_define_class namespace_module, @classname, superclass
-        @structklass = get_or_define_class @klass, :Struct, layout_superclass
         setup_class unless already_set_up
       end
+
+      def klass
+        @klass ||= get_or_define_class namespace_module, @classname, superclass
+      end
+
+      def struct_class
+        @structklass ||= get_or_define_class klass, :Struct, layout_superclass
+      end
     end
   end
 end
