@@ -1,5 +1,6 @@
 module GirFFI
   module FFIExt
+    # Extensions to FFI::Pointer
     module Pointer
       def to_ptr
         self
@@ -9,25 +10,18 @@ module GirFFI
         self
       end
 
+      def zero?
+        null?
+      end
+
       def to_object
-        gtype = GObject.type_from_instance_pointer self
-        wrap_by_gtype gtype
-      end
-
-      def wrap_by_gtype gtype
         return nil if self.null?
-        klass = Builder.build_by_gtype gtype
-        klass.direct_wrap self
+        gtype = GObject.type_from_instance_pointer self
+        Builder.build_by_gtype(gtype).direct_wrap self
       end
 
-      if RUBY_VERSION < "1.9"
-        def to_utf8
-          null? ? nil : read_string
-        end
-      else
-        def to_utf8
-          null? ? nil : read_string.force_encoding("utf-8")
-        end
+      def to_utf8
+        null? ? nil : read_string.force_encoding("utf-8")
       end
     end
   end
