@@ -5,9 +5,12 @@ module GirFFI
       def getter_name
         name.gsub(/-/, '_')
       end
+
+      def setter_name
+        "#{getter_name}="
+      end
     end
   end
 end
 
 GObjectIntrospection::IPropertyInfo.send :include, GirFFI::InfoExt::IPropertyInfo
-
