@@ -12,11 +12,6 @@ module GirFFI
           end
         end
       end
-
-      # FIXME: Remove leading colons.
-      def full_type_name
-        "::#{safe_namespace}::#{safe_name}"
-      end
     end
   end
 end
@@ -25,3 +20,4 @@ GObjectIntrospection::ICallbackInfo.send :include, GirFFI::InfoExt::SafeConstant
 GObjectIntrospection::IConstantInfo.send :include, GirFFI::InfoExt::SafeConstantName
 GObjectIntrospection::IRegisteredTypeInfo.send :include, GirFFI::InfoExt::SafeConstantName
 GObjectIntrospection::ISignalInfo.send :include, GirFFI::InfoExt::SafeConstantName
+GObjectIntrospection::IVFuncInfo.send :include, GirFFI::InfoExt::SafeConstantName
