@@ -1,8 +1,9 @@
+require 'gir_ffi/registered_type_base'
+
 module GirFFI
+  # Base module for modules representing GLib interfaces.
   module InterfaceBase
-    def gir_ffi_builder
-      self.const_get :GIR_FFI_BUILDER
-    end
+    include RegisteredTypeBase
 
     def setup_instance_method name
       gir_ffi_builder.setup_instance_method name
@@ -11,5 +12,9 @@ module GirFFI
     def wrap ptr
       ptr.to_object
     end
+
+    def to_ffitype
+      :pointer
+    end
   end
 end
