@@ -1,26 +1,24 @@
 module GirFFI
+  # Base module for modules representing GLib namespaces.
   module ModuleBase
     def method_missing method, *arguments, &block
       result = setup_method method.to_s
       return super unless result
-      self.send method, *arguments, &block
+      send method, *arguments, &block
     end
 
     def const_missing classname
       load_class(classname) || super
     end
 
-    # @deprecated Compatibility function. Remove in 0.7.0.
-    def load_class classname
-      setup_class classname
-    end
-
     def setup_class classname
       gir_ffi_builder.build_namespaced_class classname.to_s
     end
 
+    alias load_class setup_class
+
     def gir_ffi_builder
-      self.const_get :GIR_FFI_BUILDER
+      const_get :GIR_FFI_BUILDER
     end
 
     def setup_method name
