@@ -1,22 +1,6 @@
 module GirFFI
   # Base class for all generated classes of type :object.
   class ObjectBase < ClassBase
-    #
-    # Wraps a pointer retrieved from a constructor method. Here, it is simply
-    # defined as a wrapper around direct_wrap, but, e.g., InitiallyUnowned
-    # overrides it to sink the floating object.
-    #
-    # Unlike wrap, this method assumes the pointer will always be of the type
-    # corresponding to the current class, and never of a subtype.
-    #
-    # @param ptr Pointer to the object's C structure
-    #
-    # @return An object of the current class wrapping the pointer
-    #
-    def self.constructor_wrap ptr
-      direct_wrap ptr
-    end
-
     # Wrap the passed pointer in an instance of its type's corresponding class,
     # generally assumed to be a descendant of the current type.
     def self.wrap ptr
@@ -48,5 +32,13 @@ module GirFFI
     def self.to_ffitype
       :pointer
     end
+
+    def self.copy_value_to_pointer value, pointer, offset=0
+      pointer.put_pointer offset, value.to_ptr
+    end
+
+    def self.object_class
+      gir_ffi_builder.object_class
+    end
   end
 end
