@@ -4,9 +4,7 @@ module GirFFI
   class UnintrospectableTypeInfo
     attr_reader :g_type
 
-    def initialize(gtype,
-                   gir = GObjectIntrospection::IRepository.default,
-                   gobject = ::GObject)
+    def initialize gtype, gir = GObjectIntrospection::IRepository.default, gobject = GObject
       @g_type = gtype
       @gir = gir
       @gobject = gobject
@@ -21,10 +19,13 @@ module GirFFI
     end
 
     def parent
-      parent_gtype = @gobject.type_parent(@g_type)
       @gir.find_by_gtype(parent_gtype) || self.class.new(parent_gtype, @gir, @gobject)
     end
 
+    def parent_gtype
+      @parent_gtype ||= @gobject.type_parent @g_type
+    end
+
     def namespace
       parent.namespace
     end
