@@ -2,11 +2,18 @@ if RUBY_PLATFORM == 'java'
   require 'rubygems'
 end
 
-if RUBY_VERSION >= "1.9"
+begin
   require 'simplecov'
   SimpleCov.start do
     add_filter "/test/"
   end
+rescue LoadError
+end
+
+begin
+  require 'coveralls'
+  Coveralls.wear!
+rescue LoadError
 end
 
 require 'minitest/autorun'
@@ -16,6 +23,7 @@ require 'gir_ffi-base'
 require 'ffi-gobject_introspection'
 
 GObjectIntrospection::IRepository.prepend_search_path File.join(File.dirname(__FILE__), 'lib')
+
 module GObjectIntrospection
   class IRepository
     def shared_library_with_regress namespace
@@ -37,8 +45,6 @@ end
 Thread.abort_on_exception = true
 
 class Minitest::Test
-  include RR::Adapters::TestUnit
-
   def assert_defines_singleton_method klass, method, msg = nil
     method = method.to_sym
     methods = klass.singleton_methods(false).map { |name| name.to_sym }
