@@ -70,18 +70,18 @@ describe GLib::Array do
   describe ".from" do
     it "creates a GArray from an array of :gint32" do
       arr = GLib::Array.from :gint32, [3, 2, 1]
-      arr.data.read_array_of_int32(3).must_equal [3, 2, 1]
+      arr.data_ptr.read_array_of_int32(3).must_equal [3, 2, 1]
     end
 
     it "creates a GArray from an array of :gboolean" do
       arr = GLib::Array.from :gboolean, [true, false, true]
-      arr.data.read_array_of_int(3).must_equal [1, 0, 1]
+      arr.data_ptr.read_array_of_int(3).must_equal [1, 0, 1]
     end
 
     it "return its argument if given a GArray" do
       arr = GLib::Array.new :gint32
       arr.append_vals [3, 2, 1]
-      arr2 = GLib::Array.from :foo, arr
+      arr2 = GLib::Array.from :gint32, arr
       assert arr2.equal? arr
     end
 
@@ -149,8 +149,8 @@ describe GLib::Array do
 
     it "raises an error if the index is out of bounds" do
       arr = GLib::Array.from :gint32, [1, 2, 3]
-      lambda { arr.index(16) }.must_raise IndexError
-      lambda { arr.index(-1) }.must_raise IndexError
+      proc { arr.index(16) }.must_raise IndexError
+      proc { arr.index(-1) }.must_raise IndexError
     end
   end
 end
