@@ -1,4 +1,4 @@
-require File.expand_path('../gir_ffi_test_helper.rb', File.dirname(__FILE__))
+require 'gir_ffi_test_helper'
 
 describe GLib::SList do
   it "knows its element type" do
@@ -43,4 +43,32 @@ describe GLib::SList do
       assert_equal lst, lst2
     end
   end
+
+  describe "#==" do
+    it "returns true when comparing to an array with the same elements" do
+      list = GLib::SList.from :gint32, [1, 2, 3]
+
+      list.must_be :==, [1, 2, 3]
+    end
+
+    it "returns false when comparing to an array with different elements" do
+      list = GLib::SList.from :gint32, [1, 2, 3]
+
+      list.wont_be :==, [1, 2]
+    end
+
+    it "returns true when comparing to a list with the same elements" do
+      list = GLib::SList.from :gint32, [1, 2, 3]
+      other = GLib::SList.from :gint32, [1, 2, 3]
+
+      list.must_be :==, other
+    end
+
+    it "returns false when comparing to a list with different elements" do
+      list = GLib::SList.from :gint32, [1, 2, 3]
+      other = GLib::SList.from :gint32, [1, 2]
+
+      list.wont_be :==, other
+    end
+  end
 end
