@@ -1,6 +1,6 @@
 require 'gir_ffi_test_helper'
 
-require 'ffi-gobject'
+GirFFI.setup :Regress
 
 describe GObject::Value do
   describe "::Struct" do
@@ -65,7 +65,7 @@ describe GObject::Value do
     end
 
     it "handles long values" do
-      value = 0x1234_5678_9012_3456
+      value = FFI.type_size(:long) == 8 ? 0x1234_5678_9012_3456 : 0x1234_5678
       gv = GObject::Value.for_g_type GObject::TYPE_LONG
       gv.set_value value
       gv.get_long.must_equal value
@@ -93,7 +93,7 @@ describe GObject::Value do
     end
 
     it "handles ulong values" do
-      value = 0x1234_5678_9012_3456
+      value = FFI.type_size(:long) == 8 ? 0x1234_5678_9012_3456 : 0x1234_5678
       gv = GObject::Value.for_g_type GObject::TYPE_ULONG
       gv.set_value value
       gv.get_ulong.must_equal value
@@ -128,6 +128,20 @@ describe GObject::Value do
       gv.get_value.must_equal value
     end
 
+    it "unwraps an enum value" do
+      value = :value2
+      gv = GObject::Value.for_g_type Regress::TestEnum.get_gtype
+      gv.set_enum Regress::TestEnum[value]
+      gv.get_value.must_equal value
+    end
+
+    it "unwraps a flags value" do
+      value = Regress::TestFlags[:flag1] | Regress::TestFlags[:flag3]
+      gv = GObject::Value.for_g_type Regress::TestFlags.get_gtype
+      gv.set_flags value
+      gv.get_value.must_equal value
+    end
+
     it "unwraps a GType" do
       value = GObject::TYPE_STRING
       gv = GObject::Value.for_g_type GObject::TYPE_GTYPE
@@ -143,7 +157,7 @@ describe GObject::Value do
     end
 
     it "unwraps a long" do
-      value = 0x1234_5678_9012_3456
+      value = FFI.type_size(:long) == 8 ? 0x1234_5678_9012_3456 : 0x1234_5678
       gv = GObject::Value.for_g_type GObject::TYPE_LONG
       gv.set_long value
       gv.get_value.must_equal value
@@ -171,7 +185,7 @@ describe GObject::Value do
     end
 
     it "unwraps a ulong" do
-      value = 0x1234_5678_9012_3456
+      value = FFI.type_size(:long) == 8 ? 0x1234_5678_9012_3456 : 0x1234_5678
       gv = GObject::Value.for_g_type GObject::TYPE_ULONG
       gv.set_ulong value
       gv.get_value.must_equal value
