@@ -10,11 +10,7 @@ describe GObjectIntrospection::IConstantInfo do
       assert_equal :gint32, @info.constant_type.tag
     end
 
-    it "returns a value union with member :v_int32 with value 1_000_000" do
-      assert_equal 1_000_000, @info.value_union[:v_int32]
-    end
-
-    it "returns 1 as its value" do
+    it "returns 1_000_000 as its value" do
       assert_equal 1_000_000, @info.value
     end
   end
