@@ -1,6 +1,9 @@
 require 'gir_ffi_test_helper'
 
 describe GirFFI::Builders::InterfaceBuilder do
+  let(:interface_builder) { GirFFI::Builders::InterfaceBuilder.new(
+    get_introspection_data('Regress', 'TestInterface')) }
+
   describe "#build_class" do
     before do
       info = get_introspection_data 'GObject', 'TypePlugin'
@@ -16,4 +19,10 @@ describe GirFFI::Builders::InterfaceBuilder do
       assert_defines_instance_method @iface, :complete_interface_info
     end
   end
+
+  describe "#interface_struct" do
+    it "returns the interface struct type" do
+      interface_builder.interface_struct.must_equal Regress::TestInterfaceIface
+    end
+  end
 end
