@@ -19,14 +19,5 @@ describe GirFFI::InfoExt::ICallableInfo do
       end
     end
   end
-
-  describe "#return_ffi_type" do
-    it "returns the ffi type of the return type" do
-      stub(return_type_info = Object.new).to_ffitype { :some_type }
-      stub(callable_info).return_type { return_type_info }
-
-      callable_info.return_ffi_type.must_equal :some_type
-    end
-  end
 end
 
