@@ -13,6 +13,9 @@ describe GirFFI::InfoExt::ITypeInfo do
 
   let(:callback_type_info) {
     get_introspection_data('Regress', 'test_callback').args[0].argument_type }
+  let(:ghash_type_info) {
+    get_introspection_data('Regress',
+                           'test_ghash_nested_everything_return').return_type }
 
   describe "#to_ffitype" do
     it "returns an array with elements subtype and size for type :array" do
@@ -150,18 +153,6 @@ describe GirFFI::InfoExt::ITypeInfo do
     end
   end
 
-  describe "#subtype_tag_or_class" do
-    describe "without a parameter" do
-      it "returns the result of calling #tag_or_class on the first param_type" do
-        mock(elmtype_info).tag_or_class { :foo }
-
-        mock(type_info).param_type(0) { elmtype_info }
-
-        type_info.subtype_tag_or_class.must_equal :foo
-      end
-    end
-  end
-
   describe "#tag_or_class" do
     describe "for a simple type" do
       it "returns the type's tag" do
@@ -210,10 +201,10 @@ describe GirFFI::InfoExt::ITypeInfo do
       end
 
       describe "when the interface type is :object" do
-        it "returns an array with elements :pointer and built interface class" do
+        it "returns the built interface class" do
           stub(iface_info).info_type { :object }
 
-          type_info.tag_or_class.must_equal [:pointer, interface]
+          type_info.tag_or_class.must_equal interface
         end
       end
 
@@ -228,8 +219,8 @@ describe GirFFI::InfoExt::ITypeInfo do
     end
 
     describe "for a callback" do
-      it "returns :callback" do
-        callback_type_info.tag_or_class.must_equal :callback
+      it "returns the callback's wrapper class" do
+        callback_type_info.tag_or_class.must_equal Regress::TestCallback
       end
     end
 
@@ -250,6 +241,17 @@ describe GirFFI::InfoExt::ITypeInfo do
         type_info.tag_or_class.must_equal [:pointer, :void]
       end
     end
+
+    describe "for a complex nested hash type" do
+      it "returns a representeation of the nested structure" do
+        ghash_type_info.tag_or_class.must_equal(
+          [ :pointer,
+            [ :ghash,
+              :utf8,
+              [ :pointer,
+                [ :ghash, :utf8, :utf8 ] ] ] ])
+      end
+    end
   end
 
   describe "#to_callback_ffitype" do
@@ -564,4 +566,126 @@ describe GirFFI::InfoExt::ITypeInfo do
       end
     end
   end
+
+  describe "#g_type" do
+    before do
+      stub(type_info).tag { tag }
+      stub(type_info).pointer? { pointer? }
+    end
+
+    describe "for :void" do
+      let(:tag) { :void }
+      let(:pointer?) { false }
+
+      it "equals the none type" do
+        GObject.type_name(type_info.g_type).must_equal "void"
+      end
+    end
+
+    describe "for :gboolean" do
+      let(:tag) { :gboolean }
+      let(:pointer?) { false }
+
+      it "equals the gboolean type" do
+        GObject.type_name(type_info.g_type).must_equal "gboolean"
+      end
+    end
+
+    describe "for :gint32" do
+      let(:tag) { :gint32 }
+      let(:pointer?) { false }
+
+      it "equals the gint type" do
+        GObject.type_name(type_info.g_type).must_equal "gint"
+      end
+    end
+
+    describe "for :gint64" do
+      let(:tag) { :gint64 }
+      let(:pointer?) { false }
+
+      it "equals the gint64 type" do
+        GObject.type_name(type_info.g_type).must_equal "gint64"
+      end
+    end
+
+    describe "for :guint64" do
+      let(:tag) { :guint64 }
+      let(:pointer?) { false }
+
+      it "equals the guint64 type" do
+        GObject.type_name(type_info.g_type).must_equal "guint64"
+      end
+    end
+
+    describe "for pointer to :utf8" do
+      let(:tag) { :utf8 }
+      let(:pointer?) { true }
+
+      it "equals the gchararray type" do
+        GObject.type_name(type_info.g_type).must_equal "gchararray"
+      end
+    end
+
+    describe "for pointer to :ghash" do
+      let(:tag) { :ghash }
+      let(:pointer?) { true }
+
+      it "equals the GHashTable type" do
+        GObject.type_name(type_info.g_type).must_equal "GHashTable"
+      end
+    end
+
+    describe "for arrays" do
+      let(:tag) { :array }
+      let(:pointer?) { true }
+
+      describe "for pointer to GArray" do
+        before do
+          stub(type_info).zero_terminated? { false }
+          stub(type_info).array_type { :array }
+        end
+
+        it "equals the GArray type" do
+          GObject.type_name(type_info.g_type).must_equal "GArray"
+        end
+      end
+
+      describe "for a C array" do
+        before do
+          stub(type_info).zero_terminated? { false }
+          stub(type_info).array_type { :c }
+        end
+
+        it "equals the gpointer type" do
+          GObject.type_name(type_info.g_type).must_equal "gpointer"
+        end
+      end
+
+      describe "for a zero-terminated array" do
+        before do
+          stub(type_info).param_type(0) { elmtype_info }
+          stub(type_info).zero_terminated? { true }
+        end
+
+        describe "of utf8" do
+          it "equals the GStrv type" do
+            stub(elmtype_info).tag { :utf8 }
+            stub(elmtype_info).pointer? { true }
+
+            GObject.type_name(type_info.g_type).must_equal "GStrv"
+          end
+        end
+
+        describe "of filename" do
+          it "equals the GStrv type" do
+            stub(elmtype_info).tag { :filename }
+            stub(elmtype_info).pointer? { true }
+
+            GObject.type_name(type_info.g_type).must_equal "GStrv"
+          end
+        end
+      end
+    end
+  end
 end
