@@ -31,6 +31,10 @@ class Minitest::Test
     get_introspection_data(namespace, klass).find_signal name
   end
 
+  def get_vfunc_introspection_data namespace, klass, name
+    get_introspection_data(namespace, klass).find_vfunc name
+  end
+
   SAVED_MODULES = {}
 
   def save_module name
