@@ -7,6 +7,16 @@ GirFFI.setup :GIMarshallingTests
 
 # Tests generated methods and functions in the GIMarshallingTests namespace.
 describe GIMarshallingTests do
+  let(:derived_klass) { Object.const_set("DerivedClass#{Sequence.next}",
+                                         Class.new(GIMarshallingTests::Object)) }
+
+  def make_derived_instance
+    GirFFI.define_type derived_klass do |info|
+      yield info if block_given?
+    end
+    derived_klass.new
+  end
+
   describe "GIMarshallingTests::BoxedStruct" do
     let(:instance) { GIMarshallingTests::BoxedStruct.new }
 
@@ -46,7 +56,9 @@ describe GIMarshallingTests do
 
       res = GIMarshallingTests::BoxedStruct.inout bx
 
-      assert_equal 42, bx.long_
+      # TODO: Deal with the fact that bx is now invalid (at least with
+      # gobject-introspection 1.40).
+
       assert_equal 0, res.long_
     end
 
@@ -183,19 +195,31 @@ describe GIMarshallingTests do
 
   describe "GIMarshallingTests::Interface" do
     it "has a working method #test_int8_in" do
-      skip "Interfaces cannot be tested directly"
+      derived_klass.class_eval { include GIMarshallingTests::Interface }
+      instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :test_int8_in, proc {|obj, in_| obj.int = in_ }
+      end
+      instance.test_int8_in 8
+      instance.int.must_equal 8
     end
   end
 
   describe "GIMarshallingTests::Interface2" do
-    it "must be tested" do
-      skip "Interfaces cannot be tested directly"
+    it "must be tested for clashes with Interface" do
+      skip "Needs more work to determine desired behavior"
     end
   end
 
   describe "GIMarshallingTests::Interface3" do
     it "has a working method #test_variant_array_in" do
-      skip "Interfaces cannot be tested directly"
+      skip unless get_introspection_data 'GIMarshallingTests', 'Interface3'
+      derived_klass.class_eval { include GIMarshallingTests::Interface3 }
+      instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :test_variant_array_in, proc {|obj, in_|
+          obj.int = in_.to_a.first.get_byte }
+      end
+      instance.test_variant_array_in [GLib::Variant.new_byte(42)]
+      instance.int.must_equal 42
     end
   end
 
@@ -253,9 +277,13 @@ describe GIMarshallingTests do
 
     it "has a working function #full_inout" do
       ob = GIMarshallingTests::Object.new 42
+
       res = GIMarshallingTests::Object.full_inout ob
+
+      # TODO: Deal with the fact that ob is now invalid
+
       assert_instance_of GIMarshallingTests::Object, res
-      ob.wont_equal res
+      res.int.must_equal 0
     end
 
     it "has a working function #full_out" do
@@ -269,7 +297,7 @@ describe GIMarshallingTests do
     end
 
     it "has a working function #inout_same" do
-      skip "This function is only found in the header"
+      skip "This function is defined in the header but not implemented"
     end
 
     it "has a working function #none_inout" do
@@ -297,40 +325,122 @@ describe GIMarshallingTests do
     let(:instance) { GIMarshallingTests::Object.new 42 }
 
     it "has a working method #call_vfunc_with_callback" do
-      # NOTE: To call this method, the callback slot vfunc_with_callback has to
-      # be filled in the GIMarshallingTests::Object class structure. The
-      # GIMarshallingTests library doesn't do this.
-      skip "Needs vfunc setup"
+      skip unless get_method_introspection_data("GIMarshallingTests", "Object",
+                                                "call_vfunc_with_callback")
+      cb = nil
+      user_data = nil
+      result = nil
+
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_with_callback, proc { |obj, callback, callback_data|
+          cb = callback
+          user_data = callback_data.address
+          result = callback.call(42, callback_data)
+        }
+      end
+      derived_instance.call_vfunc_with_callback
+
+      user_data.must_equal 0xdeadbeef
+      # TODO: Change implementation of CallbackBase so that this becomes an
+      # instance of GIMarshallingTests::CallbackIntInt
+      cb.must_be_instance_of FFI::Function
+      result.must_equal 42
     end
 
     it "has a working method #full_in" do
-      skip "This function is only found in the header"
+      skip "This function is defined in the header but not implemented"
     end
 
     it "has a working method #get_ref_info_for_vfunc_in_object_transfer_full" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #get_ref_info_for_vfunc_in_object_transfer_none" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #get_ref_info_for_vfunc_out_object_transfer_full" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #get_ref_info_for_vfunc_out_object_transfer_none" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #get_ref_info_for_vfunc_return_object_transfer_full" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #get_ref_info_for_vfunc_return_object_transfer_none" do
-      skip "Needs vfunc setup"
-    end
-    it "has a working method #int8_in" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_in_object_transfer_full")
+      obj = nil
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_in_object_transfer_full, proc {|this, object|
+          obj = object
+        }
+      end
+      result = derived_instance.
+        get_ref_info_for_vfunc_in_object_transfer_full GIMarshallingTests::Object.get_gtype
+      result.must_equal [1, false]
+      obj.must_be_instance_of GIMarshallingTests::Object
     end
 
+    it "has a working method #get_ref_info_for_vfunc_in_object_transfer_none" do
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_in_object_transfer_none")
+      klass = nil
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_in_object_transfer_none, proc {|this, object|
+          klass = object.class
+        }
+      end
+      result = derived_instance.
+        get_ref_info_for_vfunc_in_object_transfer_none GIMarshallingTests::Object.get_gtype
+      result.must_equal [1, false]
+      klass.must_equal GIMarshallingTests::Object
+    end
+
+    it "has a working method #get_ref_info_for_vfunc_out_object_transfer_full" do
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_out_object_transfer_full")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_out_object_transfer_full, proc {|obj|
+          GIMarshallingTests::Object.new 42
+        }
+      end
+      result = derived_instance.get_ref_info_for_vfunc_out_object_transfer_full
+      result.must_equal [1, false]
+    end
+
+    it "has a working method #get_ref_info_for_vfunc_out_object_transfer_none" do
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_out_object_transfer_none")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_out_object_transfer_none, proc {|obj|
+          GIMarshallingTests::Object.new 42
+        }
+      end
+      result = derived_instance.get_ref_info_for_vfunc_out_object_transfer_none
+      result.must_equal [1, false]
+    end
+
+    it "has a working method #get_ref_info_for_vfunc_return_object_transfer_full" do
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_return_object_transfer_full")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_return_object_transfer_full, proc {|obj|
+          GIMarshallingTests::Object.new 42
+        }
+      end
+      result = derived_instance.get_ref_info_for_vfunc_return_object_transfer_full
+      result.must_equal [1, false]
+    end
+
+    it "has a working method #get_ref_info_for_vfunc_return_object_transfer_none" do
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_return_object_transfer_none")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_return_object_transfer_none, proc {|obj|
+          GIMarshallingTests::Object.new 42
+        }
+      end
+      result = derived_instance.get_ref_info_for_vfunc_return_object_transfer_none
+      result.must_equal [1, false]
+    end
+
+    it "has a working method #int8_in" do
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_in, proc {|obj, in_| obj.int = in_ }
+      end
+      derived_instance.int8_in 23
+      derived_instance.int.must_equal 23
+    end
+
+    # TODO: Check if this is fixed
+    # See https://bugzilla.gnome.org/show_bug.cgi?id=727665
     it "has a working method #int8_out" do
-      skip "Needs vfunc setup"
+      skip "Introspection data is not generated correctly"
     end
 
     it "has a working method #method" do
@@ -359,27 +469,55 @@ describe GIMarshallingTests do
     end
 
     it "has a working method #method_int8_arg_and_out_callee" do
-      skip "Needs vfunc setup"
+      skip unless get_method_introspection_data("GIMarshallingTests", "Object",
+                                                "method_int8_arg_and_out_callee")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_arg_and_out_callee, proc { |obj, arg|
+          2 * arg
+        }
+      end
+      result = derived_instance.method_int8_arg_and_out_callee 32
+      result.must_equal 64
     end
 
     it "has a working method #method_int8_arg_and_out_caller" do
-      skip "Needs vfunc setup"
+      skip unless get_method_introspection_data("GIMarshallingTests", "Object",
+                                                "method_int8_arg_and_out_caller")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_arg_and_out_caller, proc { |obj, arg|
+          2 * arg
+        }
+      end
+      result = derived_instance.method_int8_arg_and_out_caller 32
+      result.must_equal 64
     end
 
     it "has a working method #method_int8_in" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_in, proc {|obj, in_| obj.int = in_ }
+      end
+      derived_instance.method_int8_in 108
+      derived_instance.int.must_equal 108
     end
 
     it "has a working method #method_int8_out" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_out, proc {|obj| 42 }
+      end
+      derived_instance.method_int8_out.must_equal 42
     end
 
     it "has a working method #method_str_arg_out_ret" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "method_str_arg_out_ret")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_str_arg_out_ret, proc {|obj, arg| [arg, 42] }
+      end
+      derived_instance.method_str_arg_out_ret("foo").must_equal ["foo", 42]
     end
 
     it "has a working method #method_variant_array_in" do
-      skip "Needs vfunc setup"
+      skip "This function is defined in the header but not implemented"
     end
 
     it "has a working method #method_with_default_implementation" do
@@ -399,50 +537,147 @@ describe GIMarshallingTests do
     end
 
     it "has a working method #vfunc_array_out_parameter" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_array_out_parameter")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_array_out_parameter, proc {|obj|
+          [1.1, 2.2, 3.3]
+        }
+      end
+      result = derived_instance.vfunc_array_out_parameter
+      arr = result.to_a
+      arr.length.must_equal 3
+      arr[0].must_be_close_to 1.1
+      arr[1].must_be_close_to 2.2
+      arr[2].must_be_close_to 3.3
     end
 
     it "has a working method #vfunc_caller_allocated_out_parameter" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_caller_allocated_out_parameter")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_caller_allocated_out_parameter, proc {|obj|
+          "Hello!"
+        }
+      end
+      result = derived_instance.vfunc_caller_allocated_out_parameter
+      result.get_value.must_equal "Hello!"
     end
+
     it "has a working method #vfunc_meth_with_error" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_meth_with_err, proc {|object, x|
+          raise "This is not the answer!" unless x == 42
+          true
+        }
+      end
+      result = derived_instance.vfunc_meth_with_error 42
+      result.must_equal true
+
+      err = proc { derived_instance.vfunc_meth_with_error(21) }.
+        must_raise GirFFI::GLibError
+      err.message.must_equal "This is not the answer!"
+      err.domain.must_equal "gir_ffi"
+      err.code.must_equal 0
     end
+
     it "has a working method #vfunc_multiple_out_parameters" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation(
+          :vfunc_multiple_out_parameters,
+          proc {|*args| [42.1, -142.3] })
+      end
+      result = derived_instance.vfunc_multiple_out_parameters
+      result[0].must_be_close_to 42.1
+      result[1].must_be_close_to(-142.3)
     end
+
     it "has a working method #vfunc_one_out_parameter" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation(
+          :vfunc_one_out_parameter,
+          proc {|*args| 23.4 })
+      end
+      derived_instance.vfunc_one_out_parameter.
+        must_be_within_epsilon 23.4
     end
+
     it "has a working method #vfunc_out_enum" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_out_enum")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_out_enum, proc {|obj| :value2 }
+      end
+      derived_instance.vfunc_out_enum.must_equal :value2
     end
+
     it "has a working method #vfunc_return_enum" do
-      skip "Needs vfunc setup"
+      skip unless get_vfunc_introspection_data("GIMarshallingTests", "Object",
+                                               "vfunc_return_enum")
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_return_enum, proc {|obj| :value2 }
+      end
+      derived_instance.vfunc_return_enum.must_equal :value2
     end
+
     it "has a working method #vfunc_return_value_and_multiple_out_parameters" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation(
+          :vfunc_return_value_and_multiple_out_parameters,
+          proc {|*args| [42, -142, 3] })
+      end
+      derived_instance.vfunc_return_value_and_multiple_out_parameters.
+        must_equal [42, -142, 3]
     end
+
     it "has a working method #vfunc_return_value_and_one_out_parameter" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation(
+          :vfunc_return_value_and_one_out_parameter,
+          proc {|*args| [42, -142] })
+      end
+      derived_instance.vfunc_return_value_and_one_out_parameter.
+        must_equal [42, -142]
     end
+
     it "has a working method #vfunc_return_value_only" do
-      skip "Needs vfunc setup"
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_return_value_only, proc {|obj| 0x1234_5678 }
+      end
+      result = derived_instance.vfunc_return_value_only
+      result.must_equal 0x1234_5678
     end
+
     it "has a working method #vfunc_with_callback" do
-      skip "Needs vfunc setup"
+      skip unless get_method_introspection_data("GIMarshallingTests", "Object",
+                                                "vfunc_with_callback")
+
+      result = 1
+
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :vfunc_with_callback, proc { |obj, callback, callback_data|
+          callback.call(42, callback_data)
+        }
+      end
+
+      derived_instance.vfunc_with_callback proc { |val, user_data| result = val + user_data }, 23
+
+      # The current implementation of the vfunc_with_callback method currently
+      # doesn't actually call the virtual function vfunc_with_callback.
+      result.must_equal 1
+      result.wont_equal 42 + 23
     end
 
     describe "its 'int' property" do
       it "can be retrieved with #get_property" do
-        assert_equal 42, instance.get_property("int")
+        assert_equal 42, instance.get_property("int").get_value
       end
       it "can be retrieved with #int" do
         assert_equal 42, instance.int
       end
       it "can be set with #set_property" do
         instance.set_property("int", 13)
-        assert_equal 13, instance.get_property("int")
+        assert_equal 13, instance.get_property("int").get_value
       end
       it "can be set with #int=" do
         instance.int = 1
@@ -536,7 +771,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-boolean' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-boolean").must_equal false
+        instance.get_property("some-boolean").get_value.must_equal false
       end
 
       it "can be retrieved with #some_boolean" do
@@ -545,12 +780,12 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property("some-boolean", true)
-        instance.get_property("some-boolean").must_equal true
+        instance.get_property("some-boolean").get_value.must_equal true
       end
 
       it "can be set with #some_boolean=" do
         instance.some_boolean = true
-        instance.get_property("some-boolean").must_equal true
+        instance.get_property("some-boolean").get_value.must_equal true
       end
     end
 
@@ -562,21 +797,22 @@ describe GIMarshallingTests do
       end
 
       it "can be retrieved with #get_property" do
-        instance.get_property("some-boxed-glist").must_equal nil
+        instance.get_property("some-boxed-glist").get_value.must_equal nil
       end
 
       it "can be retrieved with #some_boxed_glist" do
         instance.some_boxed_glist.must_equal nil
       end
 
-      it "can be set with #set_property" do
-        instance.set_property("some-boxed-glist", [1, 2, 3])
-        instance.get_property("some-boxed-glist").to_a.must_equal [1, 2, 3]
+      it "can be set with #set_property_extended" do
+        instance.set_property_extended("some-boxed-glist", [1, 2, 3])
+        instance.some_boxed_glist.to_a.must_equal [1, 2, 3]
       end
 
       it "can be set with #some_boxed_glist=" do
         instance.some_boxed_glist = [1, 2, 3]
         instance.some_boxed_glist.to_a.must_equal [1, 2, 3]
+        instance.get_property_extended("some-boxed-glist").to_a.must_equal [1, 2, 3]
       end
     end
 
@@ -588,7 +824,7 @@ describe GIMarshallingTests do
       end
 
       it "can be retrieved with #get_property" do
-        instance.get_property("some-boxed-struct").must_equal nil
+        instance.get_property("some-boxed-struct").get_value.must_equal nil
       end
 
       it "can be retrieved with #some_boxed_struct" do
@@ -599,7 +835,7 @@ describe GIMarshallingTests do
         boxed = GIMarshallingTests::BoxedStruct.new
         boxed.long_ = 42
         instance.set_property("some-boxed-struct", boxed)
-        instance.get_property("some-boxed-struct").long_.must_equal 42
+        instance.get_property("some-boxed-struct").get_value.long_.must_equal 42
       end
 
       it "can be set with #some_boxed_struct=" do
@@ -612,7 +848,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-char' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-char").must_equal 0
+        instance.get_property("some-char").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_char" do
@@ -621,7 +857,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-char", 42
-        instance.get_property("some-char").must_equal 42
+        instance.get_property("some-char").get_value.must_equal 42
       end
 
       it "can be set with #some_char=" do
@@ -632,7 +868,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-double' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-double").must_equal 0.0
+        instance.get_property("some-double").get_value.must_equal 0.0
       end
 
       it "can be retrieved with #some_double" do
@@ -641,7 +877,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property("some-double", 3.14)
-        instance.get_property("some-double").must_equal 3.14
+        instance.get_property("some-double").get_value.must_equal 3.14
       end
 
       it "can be set with #some_double=" do
@@ -652,7 +888,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-float' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-float").must_equal 0.0
+        instance.get_property("some-float").get_value.must_equal 0.0
       end
 
       it "can be retrieved with #some_float" do
@@ -661,7 +897,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property("some-float", 3.14)
-        instance.get_property("some-float").must_be_close_to 3.14
+        instance.get_property("some-float").get_value.must_be_close_to 3.14
       end
 
       it "can be set with #some_float=" do
@@ -672,7 +908,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-int' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-int").must_equal 0
+        instance.get_property("some-int").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_int" do
@@ -681,7 +917,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-int", 4242
-        instance.get_property("some-int").must_equal 4242
+        instance.get_property("some-int").get_value.must_equal 4242
       end
 
       it "can be set with #some_int=" do
@@ -692,7 +928,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-int64' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-int64").must_equal 0
+        instance.get_property("some-int64").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_int64" do
@@ -701,7 +937,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-int64", 42_000_000_000_000
-        instance.get_property("some-int64").must_equal 42_000_000_000_000
+        instance.get_property("some-int64").get_value.must_equal 42_000_000_000_000
       end
 
       it "can be set with #some_int64=" do
@@ -712,7 +948,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-long' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-long").must_equal 0
+        instance.get_property("some-long").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_long" do
@@ -721,7 +957,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-long", 4242
-        instance.get_property("some-long").must_equal 4242
+        instance.get_property("some-long").get_value.must_equal 4242
       end
 
       it "can be set with #some_long=" do
@@ -738,7 +974,7 @@ describe GIMarshallingTests do
       end
 
       it "can be retrieved with #get_property" do
-        instance.get_property("some-object").must_be_nil
+        instance.get_property("some-object").get_value.must_be_nil
       end
 
       it "can be retrieved with #some_object" do
@@ -748,7 +984,7 @@ describe GIMarshallingTests do
       it "can be set with #set_property" do
         ob = GIMarshallingTests::Object.new 42
         instance.set_property "some-object", ob
-        instance.get_property("some-object").must_equal ob
+        instance.get_property("some-object").get_value.must_equal ob
       end
 
       it "can be set with #some_object=" do
@@ -766,16 +1002,16 @@ describe GIMarshallingTests do
       end
 
       it "can be retrieved with #get_property" do
-        instance.get_property("some-strv").must_be :==, []
+        instance.get_property("some-strv").get_value.must_be :==, []
       end
 
       it "can be retrieved with #some_strv" do
         instance.some_strv.must_be :==, []
       end
 
-      it "can be set with #set_property" do
-        instance.set_property("some-strv", ["foo", "bar"])
-        instance.get_property("some-strv").must_be :==, ["foo", "bar"]
+      it "can be set with #set_property_extended" do
+        instance.set_property_extended("some-strv", ["foo", "bar"])
+        instance.get_property("some-strv").get_value.must_be :==, ["foo", "bar"]
       end
 
       it "can be set with #some_strv=" do
@@ -786,7 +1022,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-uchar' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-uchar").must_equal 0
+        instance.get_property("some-uchar").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_uchar" do
@@ -795,7 +1031,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-uchar", 42
-        instance.get_property("some-uchar").must_equal 42
+        instance.get_property("some-uchar").get_value.must_equal 42
       end
 
       it "can be set with #some_uchar=" do
@@ -804,9 +1040,9 @@ describe GIMarshallingTests do
       end
     end
 
-    describe "its 'some-uuint' property" do
+    describe "its 'some-uint' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-uint").must_equal 0
+        instance.get_property("some-uint").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_uint" do
@@ -815,7 +1051,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-uint", 4242
-        instance.get_property("some-uint").must_equal 4242
+        instance.get_property("some-uint").get_value.must_equal 4242
       end
 
       it "can be set with #some_uint=" do
@@ -826,7 +1062,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-uint64' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-uint64").must_equal 0
+        instance.get_property("some-uint64").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_uint64" do
@@ -835,7 +1071,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-uint64", 42_000_000_000_000
-        instance.get_property("some-uint64").must_equal 42_000_000_000_000
+        instance.get_property("some-uint64").get_value.must_equal 42_000_000_000_000
       end
 
       it "can be set with #some_uint64=" do
@@ -846,7 +1082,7 @@ describe GIMarshallingTests do
 
     describe "its 'some-ulong' property" do
       it "can be retrieved with #get_property" do
-        instance.get_property("some-ulong").must_equal 0
+        instance.get_property("some-ulong").get_value.must_equal 0
       end
 
       it "can be retrieved with #some_ulong" do
@@ -855,7 +1091,7 @@ describe GIMarshallingTests do
 
       it "can be set with #set_property" do
         instance.set_property "some-ulong", 4242
-        instance.get_property("some-ulong").must_equal 4242
+        instance.get_property("some-ulong").get_value.must_equal 4242
       end
 
       it "can be set with #some_ulong=" do
@@ -872,7 +1108,7 @@ describe GIMarshallingTests do
       end
 
       it "can be retrieved with #get_property" do
-        instance.get_property("some-variant").must_be_nil
+        instance.get_property("some-variant").get_value.must_be_nil
       end
 
       it "can be retrieved with #some_variant" do
@@ -882,7 +1118,7 @@ describe GIMarshallingTests do
       it "can be set with #set_property" do
         value = GLib::Variant.new_string("Foo")
         instance.set_property "some-variant", value
-        instance.get_property("some-variant").must_equal value
+        instance.get_property("some-variant").get_value.must_equal value
       end
 
       it "can be set with #some_variant=" do
@@ -1120,6 +1356,18 @@ describe GIMarshallingTests do
     pass
   end
 
+  it "has a working function #array_in_utf8_two_in" do
+    skip unless get_introspection_data 'GIMarshallingTests', 'array_in_utf8_two_in'
+    GIMarshallingTests.array_in_utf8_two_in [-1, 0, 1, 2], "1", "2"
+    pass
+  end
+
+  it "has a working function #array_in_utf8_two_in_out_of_order" do
+    skip unless get_introspection_data 'GIMarshallingTests', 'array_in_utf8_two_in_out_of_order'
+    GIMarshallingTests.array_in_utf8_two_in_out_of_order "1", [-1, 0, 1, 2], "2"
+    pass
+  end
+
   it "has a working function #array_inout" do
     res = GIMarshallingTests.array_inout [-1, 0, 1, 2]
     res.must_be :==, [-2, -1, 0, 1, 2]
@@ -1299,23 +1547,48 @@ describe GIMarshallingTests do
   end
 
   it "has a working function #callback_multiple_out_parameters" do
-    skip "Out parameters for callbacks are not supported yet"
+    result = GIMarshallingTests.callback_multiple_out_parameters proc { |*args|
+      [12.0, 13.0]
+    }
+    result.must_equal [12.0, 13.0]
   end
 
   it "has a working function #callback_one_out_parameter" do
-    skip "Out parameters for callbacks are not supported yet"
+    result = GIMarshallingTests.callback_one_out_parameter proc { 42.0 }
+    result.must_equal 42.0
+  end
+
+  it "has a working function #callback_owned_boxed" do
+    skip unless get_introspection_data 'GIMarshallingTests', 'callback_owned_boxed'
+    a = nil
+
+    callback = proc { |box, callback_data| a = box.long_ + callback_data }
+
+    result = GIMarshallingTests.callback_owned_boxed callback, 42
+    result.must_equal 1
+    a.must_equal 43
+
+    result = GIMarshallingTests.callback_owned_boxed callback, 42
+    result.must_equal 2
+    a.must_equal 44
   end
 
   it "has a working function #callback_return_value_and_multiple_out_parameters" do
-    skip "Out parameters for callbacks are not supported yet"
+    result = GIMarshallingTests.callback_return_value_and_multiple_out_parameters proc { |*args|
+      [42, -142, 3]
+    }
+    result.must_equal [42, -142, 3]
   end
 
   it "has a working function #callback_return_value_and_one_out_parameter" do
-    skip "Out parameters for callbacks are not supported yet"
+    result = GIMarshallingTests.callback_return_value_and_one_out_parameter proc { |*args|
+      [42, -142]
+    }
+    result.must_equal [42, -142]
   end
 
   it "has a working function #callback_return_value_only" do
-    result = GIMarshallingTests.callback_return_value_only lambda { 42 }
+    result = GIMarshallingTests.callback_return_value_only proc { 42 }
     result.must_equal 42
   end
 
@@ -1497,15 +1770,21 @@ describe GIMarshallingTests do
   end
 
   it "has a working function #gbytes_full_return" do
-    skip "GBytes is not supported yet"
+    skip unless get_introspection_data 'GIMarshallingTests', 'gbytes_full_return'
+    res = GIMarshallingTests.gbytes_full_return
+    res.to_a.must_equal [0, 49, 0xFF, 51]
   end
+
   it "has a working function #gbytes_none_in" do
-    skip "GBytes is not supported yet"
+    skip unless get_introspection_data 'GIMarshallingTests', 'gbytes_none_in'
+    GIMarshallingTests.gbytes_none_in [0, 49, 0xFF, 51]
+    pass
   end
 
   it "has a working function #gclosure_in" do
     cl = GObject::RubyClosure.new { 42 }
     GIMarshallingTests.gclosure_in cl
+    pass
   end
 
   it "has a working function #gclosure_return" do
@@ -1539,8 +1818,10 @@ describe GIMarshallingTests do
     begin
       GIMarshallingTests.gerror
       flunk "Error should have been raised"
-    rescue RuntimeError => e
+    rescue GirFFI::GLibError => e
       e.message.must_equal GIMarshallingTests::CONSTANT_GERROR_MESSAGE
+      e.domain.must_equal GIMarshallingTests::CONSTANT_GERROR_DOMAIN
+      e.code.must_equal GIMarshallingTests::CONSTANT_GERROR_CODE
     end
   end
 
@@ -1548,8 +1829,10 @@ describe GIMarshallingTests do
     begin
       GIMarshallingTests.gerror_array_in [1, 2, 3]
       flunk "Error should have been raised"
-    rescue RuntimeError => e
+    rescue GirFFI::GLibError => e
       e.message.must_equal GIMarshallingTests::CONSTANT_GERROR_MESSAGE
+      e.domain.must_equal GIMarshallingTests::CONSTANT_GERROR_DOMAIN
+      e.code.must_equal GIMarshallingTests::CONSTANT_GERROR_CODE
     end
   end
 
@@ -1889,7 +2172,11 @@ describe GIMarshallingTests do
   end
 
   it "has a working function #gvalue_flat_array_round_trip" do
-    skip "Passing structs by-value is not supported yet"
+    result = GIMarshallingTests.gvalue_flat_array_round_trip 42, "42", true
+    arr = result.to_a
+    arr[0].get_value.must_equal 42
+    arr[1].get_value.must_equal "42"
+    arr[2].get_value.must_equal true
   end
 
   it "has a working function #gvalue_in" do
@@ -1906,6 +2193,13 @@ describe GIMarshallingTests do
     pass
   end
 
+  it "has a working function #gvalue_in_with_modification" do
+    skip unless get_introspection_data 'GIMarshallingTests', 'gvalue_in_with_modification'
+    gv = GObject::Value.wrap_ruby_value(42)
+    GIMarshallingTests.gvalue_in_with_modification gv
+    gv.get_value.must_equal 24
+  end
+
   it "has a working function #gvalue_in_with_type" do
     gv = GObject::Value.new
     gv.init GIMarshallingTests::SubSubObject.get_gtype
@@ -2137,6 +2431,14 @@ describe GIMarshallingTests do
     assert_equal 0x7fffffff, res
   end
 
+  it "has a working function #int_one_in_utf8_two_in_one_allows_none" do
+    skip unless get_introspection_data('GIMarshallingTests',
+                                       'int_one_in_utf8_two_in_one_allows_none')
+    GIMarshallingTests.int_one_in_utf8_two_in_one_allows_none 1, "2", "3"
+    GIMarshallingTests.int_one_in_utf8_two_in_one_allows_none 1, nil, "3"
+    pass
+  end
+
   it "has a working function #int_out_max" do
     res = GIMarshallingTests.int_out_max
     assert_equal 0x7fffffff, res
@@ -2172,6 +2474,14 @@ describe GIMarshallingTests do
     assert_equal [4, 5, 6], res
   end
 
+  it "has a working function #int_two_in_utf8_two_in_with_allow_none" do
+    skip unless get_introspection_data('GIMarshallingTests',
+                                       'int_two_in_utf8_two_in_with_allow_none')
+    GIMarshallingTests.int_two_in_utf8_two_in_with_allow_none 1, 2, "3", "4"
+    GIMarshallingTests.int_two_in_utf8_two_in_with_allow_none 1, 2, nil, nil
+    pass
+  end
+
   it "has a working function #long_in_max" do
     GIMarshallingTests.long_in_max max_long
     pass
@@ -2248,6 +2558,8 @@ describe GIMarshallingTests do
     res.must_be_instance_of GIMarshallingTests::OverridesStruct
   end
 
+  # TODO: Wait for fixed version of param_spec_in_bool to land in Debian/Ubuntu
+  # See https://bugzilla.gnome.org/show_bug.cgi?id=728409
   it "has a working function #param_spec_in_bool" do
     skip "param_spec_in_bool tests the wrong type"
     ps = GObject.param_spec_boolean "mybool", "nick", "blurb", true, :readable
@@ -2401,7 +2713,13 @@ describe GIMarshallingTests do
   end
 
   it "has a working function #test_interface_test_int8_in" do
-    skip "Needs implementation of TestInterface"
+    derived_klass.class_eval { include GIMarshallingTests::Interface }
+    instance = make_derived_instance do |info|
+      info.install_vfunc_implementation :test_int8_in, proc {|obj, in_| obj.int = in_ }
+    end
+    instance.int.must_equal 0
+    GIMarshallingTests.test_interface_test_int8_in instance, 8
+    instance.int.must_equal 8
   end
 
   it "has a working function #time_t_in" do
